/***************************************************************************************
* AUTHOR : MinZhenfei
* DATE   : 2011-2-21
* MODULE : SDKrn.C
* 
* Command: 
*	Source of IOCTRL Sample Driver
*
* Description:
*		Demonstrates communications between USER and KERNEL.
*
****************************************************************************************
* Copyright (C) 2011 MinZhenfei.
****************************************************************************************/

//#######################################################################################
//# I N C L U D E S
//#######################################################################################

#ifndef CXX_SDKRN_H
#	include "SDKrn.h"
#endif

#include "struct.h"

//////////////////////////////////////////////////////////////////////////
ULONG    g_RealServiceAddress; 
NTCREATEPROCESSEX RealNtCreateProcessEx; 
LIST_ENTRY g_List;
BOOL bHooked = FALSE, bStudyed = FALSE;
KEVENT g_KernEvent;
PKEVENT g_pUserEvent = NULL;
int g_nCanRun = 0;
//#######################################################################################
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//@@@@@@@@				D R I V E R   E N T R Y   P O I N T						 @@@@@@@@
//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//#######################################################################################
NTSTATUS
DriverEntry(IN PDRIVER_OBJECT pDriverObj, IN PUNICODE_STRING pRegistryString)
{
	NTSTATUS		status = STATUS_SUCCESS;
	UNICODE_STRING  ustrLinkName;
	UNICODE_STRING  ustrDevName;  
	PDEVICE_OBJECT  pDevObj;
	int i = 0;


    dprintf("[SDKrn] EasySys Sample Driver\r\n"
            "[SDKrn] Compiled %s %s\r\n[SDKrn] In DriverEntry : %wZ\r\n",
			__DATE__, __TIME__, pRegistryString);

	InitializeListHead(&g_List);
	// Register dispatch routines
/*
	for(i = 0; i < IRP_MJ_MAXIMUM_FUNCTION; i++)
	{
		pDriverObj->MajorFunction[i] = DispatchCommon;  
	}
*/
	pDriverObj->MajorFunction[IRP_MJ_CREATE] = DispatchCreate;
	pDriverObj->MajorFunction[IRP_MJ_CLOSE] = DispatchClose;

	// Dispatch routine for communications
	pDriverObj->MajorFunction[IRP_MJ_DEVICE_CONTROL] = DispatchDeviceControl;

	// Unload routine
	pDriverObj->DriverUnload = DriverUnload;

	// Initialize the device name.
	RtlInitUnicodeString(&ustrDevName, DEVICE_NAME);

	// Create the device object and device extension
	status = IoCreateDevice(pDriverObj, 
		0,
		&ustrDevName, 
		FILE_DEVICE_UNKNOWN,
		0,
		FALSE,
		&pDevObj);

	if(!NT_SUCCESS(status))
	{
		dprintf("[SDKrn] Error, IoCreateDevice = 0x%x\r\n", status);
		return status;
	}

    //// Get a pointer to our device extension
    //deviceExtension = (PDEVICE_EXTENSION) deviceObject->DeviceExtension;

    //// Save a pointer to the device object
    //deviceExtension->DeviceObject = deviceObject;

	if(IoIsWdmVersionAvailable(1,0x10))
	{
		//ַ֧ûԵϵͳ
		RtlInitUnicodeString(&ustrLinkName, SYMBOLIC_LINK_GLOBAL_NAME);
	}
	else
	{
		//֧
		RtlInitUnicodeString(&ustrLinkName, SYMBOLIC_LINK_NAME);
	}
	
	// Create a symbolic link to allow USER applications to access it. 
	status = IoCreateSymbolicLink(&ustrLinkName, &ustrDevName);  
	
	if(!NT_SUCCESS(status))
	{
		dprintf("[SDKrn] Error, IoCreateSymbolicLink = 0x%x\r\n", status);
		
		IoDeleteDevice(pDevObj); 
		return status;
	}	

	//
	//	TODO: Add initialization code here.
	//
	KeInitializeEvent(&g_KernEvent, SynchronizationEvent, FALSE);
    //// Tell the I/O Manger to do BUFFERED IO
    //deviceObject->Flags |= DO_BUFFERED_IO;

    //// Save the DeviveObject
    //deviceExtension->DeviceObject = deviceObject;

	dprintf("[SDKrn] DriverEntry Success\r\n");

	return STATUS_SUCCESS;
}

VOID
DriverUnload(IN PDRIVER_OBJECT pDriverObj)
{	
	UNICODE_STRING strLink;

	// Unloading - no resources to free so just return.
	dprintf("[SDKrn] Unloading...\r\n");;	

	//
	// TODO: Add uninstall code here.
	//
	while(!IsListEmpty(&g_List))
	{
		PMYDATASTRUCT a;
		PLIST_ENTRY pEntry = RemoveHeadList(&g_List);
		a = CONTAINING_RECORD(pEntry, MYDATASTRUCT, ListEntry);
		ExFreePool(a);
	}

	// Delete the symbolic link
	RtlInitUnicodeString(&strLink, SYMBOLIC_LINK_NAME);
	IoDeleteSymbolicLink(&strLink);

	// Delete the DeviceObject
	IoDeleteDevice(pDriverObj->DeviceObject);

	if (bHooked)
	{
		Unhook();
	}

	dprintf("[SDKrn] Unloaded Success\r\n");
	return;
}

NTSTATUS
DispatchCreate(IN PDEVICE_OBJECT pDevObj, IN PIRP pIrp)
{
	pIrp->IoStatus.Status = STATUS_SUCCESS;
	pIrp->IoStatus.Information = 0;

	dprintf("[SDKrn] IRP_MJ_CREATE\r\n");

	IoCompleteRequest(pIrp, IO_NO_INCREMENT);

	return STATUS_SUCCESS;
}


NTSTATUS
DispatchClose(IN PDEVICE_OBJECT pDevObj, IN PIRP pIrp)
{
	pIrp->IoStatus.Status = STATUS_SUCCESS;
	pIrp->IoStatus.Information = 0;

	dprintf("[SDKrn] IRP_MJ_CLOSE\r\n");

	IoCompleteRequest(pIrp, IO_NO_INCREMENT);

	// Return success
	return STATUS_SUCCESS;
}

NTSTATUS
DispatchCommon(IN PDEVICE_OBJECT pDevObj, IN PIRP pIrp)
{
	pIrp->IoStatus.Status = STATUS_SUCCESS;
	pIrp->IoStatus.Information = 0L;

	dprintf("[SDKrn] Common Dispatch\r\n");

	IoCompleteRequest( pIrp, 0 );

	// Return success

	return STATUS_SUCCESS;
}



NTSTATUS 
DispatchDeviceControl(IN PDEVICE_OBJECT pDevObj, IN PIRP pIrp)
{
	NTSTATUS status               = STATUS_INVALID_DEVICE_REQUEST;	 // STATUS_UNSUCCESSFUL
	PIO_STACK_LOCATION pIrpStack  = IoGetCurrentIrpStackLocation(pIrp);
	ULONG uIoControlCode          = 0;
	PVOID pIoBuffer				  = NULL;
	ULONG uInSize                 = 0;
	ULONG uOutSize                = 0;

	// Get the IoCtrl Code
	uIoControlCode = pIrpStack->Parameters.DeviceIoControl.IoControlCode;

	pIoBuffer = pIrp->AssociatedIrp.SystemBuffer;
	uInSize = pIrpStack->Parameters.DeviceIoControl.InputBufferLength;
	uOutSize = pIrpStack->Parameters.DeviceIoControl.OutputBufferLength;

	switch(uIoControlCode)
	{
		case IOCTL_HOOK:
			{			
				dprintf("[SDKrn] IOCTL_HOOK!\r\n");
				if (!bHooked)
				{
					Hook();
					bHooked = TRUE;
				}	
				// Return success
				pIrp->IoStatus.Information = 0;
				status = STATUS_SUCCESS;
			}
			break;

		case IOCTRL_UNHOOK:
			{
				// Receive data form Application
				dprintf("[SDKrn] IOCTRL_UNHOOK\r\n");

				// Do we have any data?
				if (bHooked)
				{
					Unhook();
					bHooked = FALSE;
				}
				// Return success
				pIrp->IoStatus.Information = 0;
				status = STATUS_SUCCESS;
			}
			break;

		case IOCTRL_GET_PROCESS:
			{
				// Send data to Application
				PLIST_ENTRY pList = NULL;
				PMYDATASTRUCT pStruct = NULL;

				dprintf("[SDKrn] IOCTRL_GET_PROCESS\r\n");
				pIrp->IoStatus.Information = 0;
				// If we have enough room copy the data upto the App - note copy the terminating character as well...
				if (IsListEmpty(&g_List))
				{
					KeWaitForSingleObject(&g_KernEvent, Executive, KernelMode, FALSE, NULL);
				}
				pList = RemoveHeadList(&g_List);
				pStruct = CONTAINING_RECORD(pList, MYDATASTRUCT, ListEntry);					
				RtlCopyMemory(pIoBuffer, pStruct->FilePath, strlen(pStruct->FilePath)+1);
				KdPrint(("%s",pStruct->FilePath));

				// Update the length for the App
				pIrp->IoStatus.Information = strlen( pStruct->FilePath ) + 1;

				status = STATUS_SUCCESS;
			}
			break;
			
		//
		// TODO: Add execute code here.
		//

		case IOCTRL_STUDY:
			{
				bStudyed = TRUE;
				pIrp->IoStatus.Information = 0;
				status = STATUS_SUCCESS;
			}
			break;

		case IOCTRL_UNSTUDY:
			{
				bStudyed = FALSE;
				pIrp->IoStatus.Information = 0;
				status = STATUS_SUCCESS;
			}
			break;

		case IOCTRL_INITEVENT:
			{
				HANDLE hEvent = *(PHANDLE)pIrp->AssociatedIrp.SystemBuffer;
				status = ObReferenceObjectByHandle(hEvent, EVENT_MODIFY_STATE, *ExEventObjectType, KernelMode, (PVOID*)&g_pUserEvent, NULL);						
				//ݼ
				ObDereferenceObject(g_pUserEvent);
				if (NT_SUCCESS(status))
				{
					dprintf("ʼͬ¼ɹ");
				}
				pIrp->IoStatus.Information = 0;
			}
			break;

		case IOCTRL_CANRUN:
			{
				g_nCanRun = *(int*)pIoBuffer;
				KdPrint(("g_nCanRun:%d", g_nCanRun));
				pIrp->IoStatus.Information = 0;
				status = STATUS_SUCCESS;
			}
			break;

		default:
			{
				// Invalid code sent
				dprintf("[SDKrn] Unknown IOCTL: 0x%X (%04X,%04X)\r\n", 
                                          uIoControlCode,
                                          DEVICE_TYPE_FROM_CTL_CODE(uIoControlCode),
                                          IoGetFunctionCodeFromCtlCode(uIoControlCode));
				pIrp->IoStatus.Information = 0;
				status = STATUS_INVALID_PARAMETER;	
			}
			break;
	}

	// Complete the I/O Request
	pIrp->IoStatus.Status = status;
	IoCompleteRequest(pIrp, IO_NO_INCREMENT);
	return status;
}


//
// TODO: Add your module definitions here.
//
VOID Hook() 
{ 
	ULONG Address; 	
	Address = (ULONG)KeServiceDescriptorTable->ServiceTableBase + 0x30 * 4; 	
	g_RealServiceAddress = *(ULONG*)Address; 	
	RealNtCreateProcessEx = (NTCREATEPROCESSEX)g_RealServiceAddress; 	
	DbgPrint( "Address of ReaL ADDR: 0x%08X\n", g_RealServiceAddress ); 
	DbgPrint(" Address of MyADDR: 0x%08X\n", MyNtCreateProcessEx ); 	
	// ȥڴ汣 
	__asm 
	{ 
		cli 
			mov    eax, cr0 
			and    eax, not 10000h 
			mov    cr0, eax 
	} 
	
	// ޸SSDTNtOpenProcessĵַ 
	*((ULONG*)Address) = (ULONG)MyNtCreateProcessEx; 
	
	// ָڴ汣 
	__asm 
	{ 
		mov    eax, cr0 
			or    eax, 10000h 
			mov    cr0, eax 
			sti 
	} 
} 

VOID Unhook() 
{ 
	ULONG Address; 
	Address = (ULONG)KeServiceDescriptorTable->ServiceTableBase + 0x30 * 4; 
	
	__asm 
	{ 
		cli 
			mov    eax, cr0 
			and    eax, not 10000h 
			mov    cr0, eax 
	} 
	
	// ԭSSDT 
	*((ULONG*)Address) = (ULONG)g_RealServiceAddress; 
	
	__asm 
	{ 
		mov    eax, cr0 
			or    eax, 10000h 
			mov    cr0, eax 
			sti 
	} 
	
	dprintf("Unhook"); 
 } 


NTSTATUS __stdcall MyNtCreateProcessEx(OUT PHANDLE ProcessHandle, 
									   IN ACCESS_MASK DesiredAccess, 
									   IN POBJECT_ATTRIBUTES ObjectAttributes OPTIONAL, 
									   IN HANDLE ParentProcess, 
									   IN BOOLEAN InheritObjectTable, 
									   IN HANDLE SectionHandle OPTIONAL, 
									   IN HANDLE DebugPort OPTIONAL, 
									   IN HANDLE ExceptionPort OPTIONAL, 
									   IN BOOLEAN InJob) 
{ 	
	PFILE_OBJECT FileObject;
	PVOID SectionObject;
	POBJECT_NAME_INFORMATION wcFilePath;
	ANSI_STRING ansiFilePath;
	int flag = 0;
//	CHAR cFilePath[MAX_PATH] = {0};

	if (NT_SUCCESS(ObReferenceObjectByHandle(SectionHandle,0,NULL,KernelMode,&SectionObject,NULL)))//ȡļ
	{
		PFILE_OBJECT FileObject;
		PVOID SectionObject;
		POBJECT_NAME_INFORMATION wcFilePath;
		
		if (NT_SUCCESS(ObReferenceObjectByHandle(SectionHandle,0,NULL,KernelMode,&SectionObject,NULL)))//ȡļ
		{
			_try
			{
				FileObject = (PFILE_OBJECT)(*((ULONG *)SectionObject + 5)); // PSEGMENT
				FileObject = *(PFILE_OBJECT *)FileObject; // CONTROL_AREA
				FileObject = *(PFILE_OBJECT *)((ULONG)FileObject + 36); // FILE_OBJECT
				
				ObReferenceObjectByPointer((PVOID)FileObject, 0, NULL, KernelMode); 
				
				if (IoQueryFileDosDeviceName(FileObject,&wcFilePath)==STATUS_SUCCESS)//ȡļӦļDos豸,ȫ·
				{
					PMYDATASTRUCT mystruct = (PMYDATASTRUCT)ExAllocatePool(NonPagedPool, sizeof(MYDATASTRUCT));				
					RtlUnicodeStringToAnsiString(&ansiFilePath, &wcFilePath->Name, TRUE);
					memcpy(mystruct->FilePath, ansiFilePath.Buffer, ansiFilePath.Length);
					mystruct->FilePath[ansiFilePath.Length] = '\0';
					KdPrint(("%s", mystruct->FilePath));

					if(IsListEmpty(&g_List))
					{
						InsertTailList(&g_List, &mystruct->ListEntry);	
						KeSetEvent(&g_KernEvent, IO_NO_INCREMENT, FALSE);
					}
					else
					{
						InsertTailList(&g_List, &mystruct->ListEntry);	
					}
					ExFreePool(wcFilePath);//IoQueryFileDosDeviceNameȡOBJECT_NAME_INFORMATION Ҫֶͷ
				}
				else
				{
					KdPrint(("[ScanFunc] MmCreatePeb is called by [Path=%ws]\r\n",FileObject->FileName.Buffer));
				}  
				ObDereferenceObject(FileObject); 
			}
			_except(EXCEPTION_EXECUTE_HANDLER)
			{
				dprintf("catch error\r\n");
			}
			ObDereferenceObject(SectionObject);
			RtlFreeAnsiString(&ansiFilePath);
		}
	}   
	//ѧϰģʽ
	if (bStudyed)
	{
		KdPrint(("ѧϰģʽ~"));
		return RealNtCreateProcessEx(ProcessHandle, 
									 DesiredAccess, 
									 ObjectAttributes, 
									 ParentProcess, 
									 InheritObjectTable, 
									 SectionHandle, 
									 DebugPort, 
									 ExceptionPort, 
									 InJob);
	}
	else //ڰȫģʽ
	{
		KeWaitForSingleObject(g_pUserEvent, Executive, KernelMode, FALSE, NULL);
		if (g_nCanRun)
		{
			KdPrint(("ڰȫģʽ~"));
			return RealNtCreateProcessEx(ProcessHandle, 
											DesiredAccess, 
											ObjectAttributes, 
											ParentProcess, 
											InheritObjectTable, 
											SectionHandle, 
											DebugPort, 
											ExceptionPort, 
											InJob);
		}
		else
		{
			//ʹؾΪգ޷
			ProcessHandle = NULL; 
			return STATUS_ACCESS_DENIED;   
		}
	}
 } 
/* EOF */
